"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const puppeteer_core_1 = __importDefault(require("puppeteer-core"));
function default_1(inputEntries, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!options.selectors || options.selectors.length === 0) {
            return inputEntries;
        }
        const browser = yield puppeteer_core_1.default.launch({
            executablePath: options.chromeExePath,
        });
        try {
            const page = yield browser.newPage();
            yield page.setViewport({ width: 1280, height: 1800 });
            // create util functions with inner scope
            const formatUrl = (url) => {
                return url;
            };
            const goTo = (options) => __awaiter(this, void 0, void 0, function* () {
                const url = formatUrl(options.url);
                console.log("navigating to " + url);
                page.goto(url);
                let response_event_occurred = false;
                const response_handler = function () {
                    response_event_occurred = true;
                };
                const response_watcher = new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        if (!response_event_occurred) {
                            resolve(true);
                        }
                        else {
                            setTimeout(function () {
                                resolve(true);
                            }, 30000);
                        }
                        page.removeListener("response", response_handler);
                    }, 500);
                });
                page.on("response", response_handler);
                var navigation_watcher = page.waitForNavigation();
                yield Promise.race([response_watcher, navigation_watcher]);
                yield waitFor(options.waitFor || "body");
            });
            const createVerifyElementFunction = (options) => {
                const { url, getters, waitFor, initialValue } = options;
                return () => __awaiter(this, void 0, void 0, function* () {
                    if (page.url() !== url) {
                        yield goTo({ url, waitFor });
                    }
                    else {
                        return initialValue;
                    }
                    let _element;
                    for (let i = 0; i < getters.length; i++) {
                        const getter = getters[i];
                        _element = yield getter(_element);
                    }
                    return _element;
                });
            };
            const getBody = () => __awaiter(this, void 0, void 0, function* () {
                const body = yield page.$("body");
                return createBrowserData({
                    url: page.url(),
                    getters: [() => page.$("body")],
                    initialValue: body,
                });
            });
            const waitFor = (selector) => __awaiter(this, void 0, void 0, function* () {
                yield page.waitForSelector(selector, { timeout: 5000 });
            });
            const processSelector = (selector, _options) => __awaiter(this, void 0, void 0, function* () {
                if (!selector.value) {
                    throw new Error('selector "' + selector.selector + '" has no value');
                }
                const { url, getters, waitFor } = _options;
                const element = yield createVerifyElementFunction(_options)();
                const entries = yield element.$$(selector.selector);
                if (entries.length === 0) {
                    console.error('No entries found for "' + selector.selector + '" on ' + page.url());
                }
                let rtn = {};
                for (let i = 0; i < entries.length; i++) {
                    const entry = entries[i];
                    const browserData = createBrowserData({
                        url,
                        getters: [
                            ...getters,
                            scoped((i) => (element) => __awaiter(this, void 0, void 0, function* () {
                                const entries = yield element.$$(selector.selector);
                                return entries[i];
                            }), [i]),
                        ],
                        initialValue: entry,
                        waitFor,
                    });
                    const value = yield selector.value(browserData);
                    // normalize the entries
                    for (let type in value) {
                        const typeEntries = value[type];
                        for (let name in typeEntries) {
                            let value = typeEntries[name];
                            if (!value) {
                                console.error("no url provided for " + name);
                                delete typeEntries[name];
                                continue;
                            }
                            if (!value.match(/^[a-z]*:\/\//)) {
                                // it's a local reference
                                if (value.startsWith("#")) {
                                    const fileName = _options.url.match(/([^/]*)$/)[1];
                                    value = fileName + value;
                                }
                                value = value.replace(/^\.?\//, "");
                                const base = _options.url.replace(/[^\/]*^/, "");
                                value = base + value;
                                typeEntries[name] = value;
                            }
                        }
                    }
                    rtn = union(rtn, value);
                }
                return rtn;
            });
            const union = (a, b) => {
                a = a || {};
                b = b || {};
                const anyA = a;
                Object.entries(b).forEach(([key, bEntries]) => {
                    if (!anyA[key]) {
                        anyA[key] = {};
                    }
                    const aEntries = anyA[key];
                    anyA[key] = Object.assign(Object.assign({}, aEntries), bEntries);
                });
                return anyA;
            };
            const createBrowserData = (options) => {
                const { url, getters, initialValue, waitFor } = options;
                let element = initialValue;
                const verifyElement = createVerifyElementFunction(options);
                return {
                    outerHTML: () => __awaiter(this, void 0, void 0, function* () {
                        if (!element) {
                            return null;
                        }
                        element = yield verifyElement();
                        const rtn = yield (yield element.getProperty("outerHTML")).jsonValue();
                        return rtn;
                    }),
                    innerText: () => __awaiter(this, void 0, void 0, function* () {
                        if (!element) {
                            return null;
                        }
                        element = yield verifyElement();
                        const rtn = yield (yield element.getProperty("innerText")).jsonValue();
                        return rtn;
                    }),
                    attr: (key) => __awaiter(this, void 0, void 0, function* () {
                        if (!element) {
                            return null;
                        }
                        element = yield verifyElement();
                        const rtn = yield (yield element.getProperty(key)).jsonValue();
                        return rtn;
                    }),
                    single: (selector) => __awaiter(this, void 0, void 0, function* () {
                        element = yield verifyElement();
                        const value = yield element.$(selector);
                        if (value) {
                            return createBrowserData({
                                url,
                                getters: [...getters, (element) => element.$(selector)],
                                initialValue: value,
                            });
                        }
                        else {
                            throw new Error('"single" selector did not match results: ' +
                                selector +
                                " on " +
                                page.url());
                        }
                    }),
                    all: (selector) => __awaiter(this, void 0, void 0, function* () {
                        const values = yield element.$$(selector);
                        const rtn = [];
                        for (let i = 0; i < values.length; i++) {
                            const browserData = createBrowserData({
                                url,
                                getters: [
                                    ...getters,
                                    scoped((i) => (element) => __awaiter(this, void 0, void 0, function* () {
                                        const values = yield element.$$(selector);
                                        return values[i];
                                    }), [i]),
                                ],
                                initialValue: values[i],
                            });
                            rtn.push(browserData);
                        }
                        return rtn;
                    }),
                    addTo: (selector, value) => __awaiter(this, void 0, void 0, function* () {
                        value = value || {};
                        const rtn = yield processSelector(selector, options);
                        return union(value, rtn);
                    }),
                    goTo: (url, waitFor) => __awaiter(this, void 0, void 0, function* () {
                        if (typeof url !== "string") {
                            throw new Error("Invalud url: " + url);
                        }
                        yield goTo({ url, waitFor });
                        const rtn = yield getBody();
                        return rtn;
                    }),
                };
            };
            const entries = union({}, inputEntries);
            for (let i = 0; i < options.selectors.length; i++) {
                const selector = options.selectors[i];
                yield goTo({ url: selector.url, waitFor: selector.waitFor });
                const initialValue = yield page.$("body");
                const value = yield processSelector(options.selectors[i], {
                    url: selector.url,
                    initialValue,
                    getters: [() => page.$("body")],
                    waitFor: selector.waitFor,
                });
                union(entries, value);
            }
            options.entries = union(options.entries, entries);
            yield browser.close();
            return entries;
        }
        catch (e) {
            console.error(e);
            yield browser.close();
            throw e;
        }
    });
}
exports.default = default_1;
function scoped(func, args) {
    return func(...args);
}
//# sourceMappingURL=browser-selector.js.map