"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sql_wasm_1 = __importDefault(require("sql-wasm"));
const path_1 = require("path");
const fs_extra_1 = __importStar(require("fs-extra"));
const rimraf_1 = __importDefault(require("rimraf"));
const browser_selector_1 = __importDefault(require("./browser-selector"));
// the following doesn't have a type definition
const recursiveCopy = require("recursive-copy");
function escape(value) {
    return value.replace(/'/g, "''");
}
function default_1(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { iconsDirPath, docsPath, excludePathPrefix, includePathPrefix, docsetIdentifier, isJavascriptEnabled, docsetPlatformFamily, fallbackUrl, outputPath, addToTopDirPath, addToBottomDirPath, selectors, } = options;
        let entries = options.entries;
        const indexFileName = options.indexFileName || "index.html";
        const indexFilePath = path_1.normalize(options.indexFileDirPath
            ? path_1.join(options.indexFileDirPath, indexFileName)
            : indexFileName).replace(/\\/g, "/");
        const docsetFileName = docsetIdentifier + ".docset";
        const outputBasePath = path_1.join(outputPath, docsetFileName);
        const outputContentsPath = path_1.join(outputBasePath, "Contents");
        const outputResourcesPath = path_1.join(outputContentsPath, "Resources");
        const outputDocsPath = path_1.join(outputResourcesPath, "Documents");
        const docsetName = options.docsetName || docsetIdentifier;
        const dryRun = options.dryRun;
        const log = (...args) => {
            if (options.logToConsole) {
                console.log(...args);
            }
        };
        if (dryRun) {
            if (selectors) {
                entries = yield browser_selector_1.default(entries, options);
            }
            console.log(entries);
            return;
        }
        else {
            try {
                log("deleting ", outputPath);
                rimraf_1.default.sync(outputPath);
            }
            catch (e) {
                console.error(e);
            }
            if (selectors) {
                entries = yield browser_selector_1.default(entries, options);
            }
            log("copying from " + docsPath + " to " + outputDocsPath);
            yield recursiveCopy(docsPath, outputDocsPath);
            log("creating index");
            const wasmPath = require.resolve("sql-wasm/dist/sqlite3.wasm");
            const SQL = yield sql_wasm_1.default({ wasmUrl: wasmPath });
            const db = new SQL.Database();
            const commands = [
                `CREATE TABLE searchIndex(id INTEGER PRIMARY KEY, name TEXT, type TEXT, path TEXT);`,
                `CREATE UNIQUE INDEX anchor ON searchIndex (name, type, path);`,
            ];
            const fileRefs = [];
            Object.entries(entries).forEach(([type, entries]) => {
                Object.entries(entries).forEach(([name, path]) => {
                    let _path = path;
                    if (excludePathPrefix && _path.startsWith(excludePathPrefix)) {
                        _path = _path.substring(excludePathPrefix.length);
                    }
                    if (includePathPrefix) {
                        _path = path_1.normalize(path_1.join(includePathPrefix, _path));
                    }
                    _path = _path.replace(/[\\/]#/, "#").replace(/\\/g, "/");
                    if (_path.endsWith("/")) {
                        _path = _path + indexFileName;
                    }
                    if (!_path.match(/^\.?\//) && options.indexFileDirPath) {
                        _path = path_1.normalize(path_1.join(options.indexFileDirPath, _path));
                    }
                    if (_path.startsWith("/")) {
                        _path = _path.replace(/^\//, "");
                    }
                    _path = _path.replace(/\\/g, "/");
                    fileRefs.push({ type: type, path: _path, name });
                    commands.push(`INSERT OR IGNORE INTO searchIndex(name, type, path) VALUES ('${escape(name)}', '${escape(type)}', '${escape(_path)}');`);
                });
            });
            for (let i = 0; i < commands.length; i++) {
                yield db.exec(commands[i]);
            }
            // save to file
            const data = db.export();
            const buffer = Buffer.from(data);
            yield fs_extra_1.default.writeFile(path_1.join(outputResourcesPath, "docSet.dsidx"), buffer);
            // create info.plist
            log("creating info.plist");
            const infoPlistData = `
      <?xml version="1.0" encoding="UTF-8"?>
      <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
      <plist version="1.0">
      <dict>
        <key>CFBundleIdentifier</key>
        <string>${docsetIdentifier}</string>
        <key>CFBundleName</key>
        <string>${docsetName}</string>
        <key>DocSetPlatformFamily</key>
        <string>${docsetPlatformFamily}</string>
        <key>isDashDocset</key>
        <true/>
        <key>dashIndexFilePath</key>
        <string>${indexFilePath}</string>
        <key>isJavaScriptEnabled</key><${isJavascriptEnabled ? "true" : "false"}/>${fallbackUrl
                ? `
        <key>DashDocSetFallbackURL</key>
        <string>${fallbackUrl}</string>`
                : ""}
      </dict>
      </plist>
    `;
            yield fs_extra_1.default.writeFile(path_1.join(outputContentsPath, "info.plist"), infoPlistData);
            // icons
            if (iconsDirPath) {
                log("copying icons...");
                const iconPath = path_1.join(iconsDirPath, "icon.png");
                const iconExists = fs_extra_1.existsSync(iconPath);
                if (iconExists) {
                    yield fs_extra_1.copy(iconPath, path_1.join(outputBasePath, "icon.png"));
                }
                else {
                    console.error(iconPath + " does not exist");
                }
                const icon2xPath = path_1.join(iconsDirPath, "icon@2x.png");
                const icon2xExists = fs_extra_1.existsSync(iconPath);
                if (icon2xExists) {
                    yield fs_extra_1.copy(icon2xPath, path_1.join(outputBasePath, "icon@2x.png"));
                }
                else {
                    console.error(icon2xPath + " does not exist");
                }
            }
            // prefix / suffix
            const checkAdds = (path, top) => {
                const base = top ? addToTopDirPath : addToBottomDirPath;
                if (!base) {
                    return;
                }
                const deltaPath = path ? path_1.join(base, path) : base;
                const outputBasePathWithPrefix = path
                    ? path_1.join(outputDocsPath, path)
                    : outputDocsPath;
                const docsContents = fs_extra_1.default.readdirSync(outputBasePathWithPrefix);
                for (let j = 0; j < docsContents.length; j++) {
                    const name = docsContents[j];
                    const outputFilePath = path_1.join(outputBasePathWithPrefix, name);
                    const stats = fs_extra_1.default.statSync(outputFilePath);
                    if (stats.isFile()) {
                        const ext = name.match(/\.([^.]+)$/)[1];
                        let matchPath;
                        // exact match
                        const exactMatchPath = path_1.join(deltaPath, name);
                        if (fs_extra_1.existsSync(exactMatchPath)) {
                            matchPath = exactMatchPath;
                        }
                        if (!matchPath) {
                            const dirWildcardMatch = path_1.join(deltaPath, "*." + ext);
                            if (fs_extra_1.existsSync(dirWildcardMatch)) {
                                matchPath = dirWildcardMatch;
                            }
                        }
                        if (!matchPath) {
                            const ext = name.match(/\.([^.]+)$/)[1];
                            const allWildcardMatch = path_1.join(base, "_all", "*." + ext);
                            if (fs_extra_1.existsSync(allWildcardMatch)) {
                                matchPath = allWildcardMatch;
                            }
                        }
                        if (matchPath) {
                            // manually do these changes
                            const srcData = fs_extra_1.default.readFileSync(outputFilePath, {
                                encoding: "utf8",
                            });
                            const deltaData = fs_extra_1.default.readFileSync(matchPath, {
                                encoding: "utf8",
                            });
                            const newData = top
                                ? deltaData + "\n" + srcData
                                : srcData + "\n" + deltaData;
                            fs_extra_1.default.writeFileSync(outputFilePath, newData, { encoding: "utf8" });
                        }
                    }
                    else {
                        checkAdds(path ? path_1.join(path, name) : name, top);
                    }
                }
            };
            checkAdds(undefined, true);
            checkAdds(undefined, false);
            fileRefs.forEach(({ type, name, path }) => {
                if (path.startsWith("#")) {
                    path = path_1.join(outputDocsPath, indexFilePath) + path;
                }
                // make sure the file is valid
                path = path_1.normalize(path_1.join(outputDocsPath, path));
                const pathToCheck = path.replace(/#.*/, "");
                if (!fs_extra_1.default.existsSync(pathToCheck)) {
                    throw new Error(`${path} not found`);
                }
                path = "file://" + encodeURI(path.replace(/\\/g, "/"));
                console.log(type + " > " + name + "\n\t" + path);
            });
        }
    });
}
exports.default = default_1;
//# sourceMappingURL=docset-creator.js.map